
#include "helper_functions.hpp"
#include <algorithm>
#include <chrono>
#include <ctime>
#include <iostream>
#include <string>
#include <vector>

static unsigned long mean_vector_size = 10000;
static unsigned long search_vector_size = 10000;
static unsigned long matrix_height_a = 100;
static unsigned long matrix_width_a = 100;
static unsigned long matrix_height_b = 100;
static unsigned long matrix_width_b = 100;

void matrix_mul(matrix &result, matrix &a, matrix &b) {
    unsigned long n = a.size();    // hoehe a
    unsigned long m = a[0].size(); // breite a /hoehe b
    unsigned long l = b[0].size(); // breite b
    if (m == b.size()) {
        for (unsigned long i = 0; i < n; i++)
            for (unsigned long k = 0; k < l; k++)
                for (unsigned long j = 0; j < m; j++) {
                    result[i][j] += a[i][k] * b[k][j];
                }
    } else {
        std::cout << "Error: matricies dont have the right dimensions " << std::endl;
        exit(EXIT_FAILURE);
    }
}

void benchmark_search_cpp() {

    std::vector<int> vec;
    std::vector<bool> vec2(search_vector_size);
    std::clock_t end;
    std::clock_t start;

    vec = my_fill(search_vector_size);
    start = std::clock();

    for (unsigned long i = 0; i < search_vector_size; i++) {
        if (std::find(vec.begin(), vec.end(), i) != vec.end()) {
            // THis was added so that the compiler doesnt
            // remove this, when compiling with O3,
            // otherwise unused part of the code
            vec2.push_back(true);
        };
    }

    end = std::clock();
    std::cout << "result: "<< vec2.size() << std::endl;
    ;
    print_time("vector_search", start, end);
}

void benchmark_matrix_mul_cpp() {
    std::clock_t start;
    std::clock_t end;

    matrix c = generate_matrix(100, 100);

    matrix result(c[0].size(), std::vector<float>(c.size(), 0));
    start = std::clock();
    matrix_mul(result, c, c);
    end = std::clock();
    print_time("matrix_mul", start, end);
}

void benchmark_mean_cpp() {

    std::vector<int> vec;
    float result;
    std::clock_t end;
    std::clock_t start;

    vec = my_fill(mean_vector_size);
    result = 0;
    start = std::clock();
    for (unsigned long i = 0; i < mean_vector_size; i++) {
        result += vec[i];
    }
    result = result / mean_vector_size;
    std::cout << "result: "<< result << std::endl;
    end = std::clock();
    print_time("vector_mean", start, end);
}

int main() {
    
    std::cout << "========================================================" << std::endl;
    std::cout << "Matrix A dims: " << matrix_height_a << " " << matrix_width_a << std::endl;
    std::cout << "Matrix B dims: " << matrix_height_b << " " << matrix_width_b << std::endl;
    std::cout << "Mean vector size: " << mean_vector_size << std::endl;
    std::cout << "search vector size: " << search_vector_size << std::endl;
    std::cout << "========================================================" << std::endl << std::endl;

    benchmark_matrix_mul_cpp();
    std::cout << "========================================================" << std::endl << std::endl;
    benchmark_search_cpp();
    std::cout << "========================================================" << std::endl << std::endl;
    benchmark_mean_cpp();
    std::cout << "========================================================" << std::endl;
    std::cout << "benchmarks done" << std::endl;
    return 1;
}
